/*
 * Gimbal
 * Copyright (C) 2024 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.gimbal.mixin;

import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.StairsBlock;
import net.minecraft.item.BlockItem;
import net.minecraft.item.Item;
import net.minecraft.item.ItemPlacementContext;
import net.moonleay.gimbal.editor.ServerEditorManager;
import net.moonleay.gimbal.editor.state.mode.Capability;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Redirect;

import java.util.UUID;

import static net.minecraft.block.StairsBlock.*;

@Mixin(BlockItem.class)
public abstract class ReplaceStateUpdaterMixin extends Item {

    public ReplaceStateUpdaterMixin(Settings settings) {
        super(settings);
    }

    @Shadow
    protected abstract boolean place(ItemPlacementContext context, BlockState state);

    @Redirect(method = "place(Lnet/minecraft/item/ItemPlacementContext;)Lnet/minecraft/util/ActionResult;", at = @At(value = "INVOKE", target = "Lnet/minecraft/item/BlockItem;place(Lnet/minecraft/item/ItemPlacementContext;Lnet/minecraft/block/BlockState;)Z"))
    private boolean func(BlockItem instance, ItemPlacementContext context, BlockState state) {
        if (context.getPlayer() == null)
            return this.place(context, state);
        UUID id = context.getPlayer().getGameProfile().getId();
        if (!ServerEditorManager.INSTANCE.shouldPlayer(id, Capability.REPLACE))
            return this.place(context, state);
        BlockState oldState = context.getWorld().getBlockState(context.getBlockPos());
        Block targetBl = oldState.getBlock();
        if (state.getBlock() instanceof StairsBlock && targetBl instanceof StairsBlock targetStair) {
            // Block and item is stairs, parse Stairs data
            return this.place(context, copyStairState(oldState, instance));
        }
        return this.place(context, state);
    }


    @Unique
    public BlockState copyStairState(BlockState targetState, BlockItem item) {
        BlockState blockState = item.getBlock().getDefaultState()
                .with(FACING, targetState.get(FACING))
                .with(HALF, targetState.get(HALF))
                .with(WATERLOGGED, targetState.get(WATERLOGGED));
        return blockState.with(SHAPE, targetState.get(SHAPE));
    }
}
